/*
 * libpasori_com_libusb_multi.c
 * 
 * The BSD License:
 *
 * Copyright (c) 2008, makai
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 * 3 .Neither the name of the author nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "libpasori_liblocal.h"
#include <usb.h>

#define PASORIUSB_VENDOR    0x054c
#define PASORIUSB_PRODUCT   0x01bb

#define MAX_PASORI_OPEN_MULTI    10

void pasori_close_multi(int n_pasori, pasori** pp) {
    int i;

    if(!pp) return;
    for (i = 0; i < n_pasori; i++) {
        usb_release_interface(
            (*pp)->dh,
            (*pp)->dev->config->interface->altsetting->bInterfaceNumber);
        usb_close((*pp)->dh);
        free(*pp);

        pp++;
    }
}

int pasori_open_multi(pasori ***ppp) {
    struct usb_bus *bus;
    struct usb_device *dev;
    pasori **pp;
    int n_pasori;
    int i;

    n_pasori = 0;
    pp = (pasori **)malloc(sizeof(pasori) * MAX_PASORI_OPEN_MULTI);
    *ppp = pp;

    usb_init();
#ifdef DEBUG
    usb_set_debug(255);
#else
    usb_set_debug(0);
#endif
    usb_find_busses();
    usb_find_devices();

    for(bus = usb_get_busses(); bus ; bus = bus->next) {
        for(dev = bus->devices; dev ; dev = dev->next) {
            Log("check for %04x:%04x\n",dev->descriptor.idVendor,dev->descriptor.idProduct); /* debug */
            if(dev->descriptor.idVendor == PASORIUSB_VENDOR &&
               dev->descriptor.idProduct== PASORIUSB_PRODUCT) {
                *pp = (pasori *)malloc(sizeof(pasori));
                (*pp)->dev = dev;
                pp++;
                if (n_pasori < MAX_PASORI_OPEN_MULTI) {
                    n_pasori++;
                } else {
                    goto finish;
                }
            }
        }
    }
    if (0 == n_pasori) {
        Log("pasori not found in USB BUS");
        return n_pasori;
    }
finish:
    pp = *ppp;
    for (i = 0; i < n_pasori; i++) {
        (*pp)->dh = usb_open((*pp)->dev);

        if(usb_set_configuration((*pp)->dh,1)) {
            /* error */
            pasori_close_multi(n_pasori, pp);
            return n_pasori;
        }
        if(usb_claim_interface(
                (*pp)->dh,
                (*pp)->dev->config->interface->altsetting->bInterfaceNumber)) {
            /* error */
            pasori_close_multi(n_pasori, pp);
            return n_pasori;
        }

        pp++;
    }

    return n_pasori;
}

/* EOF */
